@@ -413,6 +413,8 @@ bool CResize::setWindowSize(FAR struct nxgl_size_s *size)
 
 void CResize::updateSizeLabel(FAR struct nxgl_size_s &windowSize)
 {
+  int ret;
+
   // Do nothing if the size has not changed
 
   struct nxgl_size_s labelSize;
@@ -424,8 +426,8 @@ void CResize::updateSizeLabel(FAR struct nxgl_size_s &windowSize)
     }
 
   FAR char *str;
-  asprintf(&str, " %4d x %-4d ", windowSize.w, windowSize.h);
-  if (str == (FAR char *)0)
+  ret = asprintf(&str, " %4d x %-4d ", windowSize.w, windowSize.h);
+  if (ret < 0)
     {
       twmerr("ERROR: Failed to get size string\n");
       return;
