@@ -122,6 +122,15 @@ static FAR const char * const g_resetflag[] =
 };
 #endif
 
+/****************************************************************************
+ * Private Types
+ ****************************************************************************/
+
+#if defined(CONFIG_RPMSG) && !defined(CONFIG_NSH_DISABLE_RPMSG)
+typedef CODE int (*cmd_rpmsg_cb_t)(FAR int *cmd, FAR unsigned long * val,
+                                   FAR char **argv);
+#endif
+
 /****************************************************************************
  * Public Functions
  ****************************************************************************/
@@ -510,39 +519,30 @@ int cmd_reset_cause(FAR struct nsh_vtbl_s *vtbl, int argc, FAR char **argv)
 #endif
 
 /****************************************************************************
- * Name: cmd_rptun
+ * Name: cmd_rpmsg
  ****************************************************************************/
 
-#if defined(CONFIG_RPTUN) && !defined(CONFIG_NSH_DISABLE_RPTUN)
-static int cmd_rptun_once(FAR struct nsh_vtbl_s *vtbl,
-                          FAR const char *path, FAR char **argv)
+#if defined(CONFIG_RPMSG) && !defined(CONFIG_NSH_DISABLE_RPMSG)
+static int cmd_rpmsg_once(FAR struct nsh_vtbl_s *vtbl,
+                          FAR const char *path, FAR char **argv,
+                          cmd_rpmsg_cb_t rpmsg_cb)
 {
-  struct rptun_ping_s ping;
+#ifdef CONFIG_RPMSG_PING
+  struct rpmsg_ping_s ping;
+#endif
   unsigned long val = 0;
   int cmd;
   int fd;
 
-  if (strcmp(argv[1], "start") == 0)
+  if (strcmp(argv[1], "panic") == 0)
     {
-      cmd = RPTUNIOC_START;
-    }
-  else if (strcmp(argv[1], "stop") == 0)
-    {
-      cmd = RPTUNIOC_STOP;
-    }
-  else if (strcmp(argv[1], "reset") == 0)
-    {
-      val = atoi(argv[3]);
-      cmd = RPTUNIOC_RESET;
-    }
-  else if (strcmp(argv[1], "panic") == 0)
-    {
-      cmd = RPTUNIOC_PANIC;
+      cmd = RPMSGIOC_PANIC;
     }
   else if (strcmp(argv[1], "dump") == 0)
     {
-      cmd = RPTUNIOC_DUMP;
+      cmd = RPMSGIOC_DUMP;
     }
+#ifdef CONFIG_RPMSG_PING
   else if (strcmp(argv[1], "ping") == 0)
     {
       if (argv[3] == 0 || argv[4] == 0 ||
@@ -557,9 +557,14 @@ static int cmd_rptun_once(FAR struct nsh_vtbl_s *vtbl,
       ping.ack   = atoi(argv[5]);
       ping.sleep = atoi(argv[6]);
 
-      cmd = RPTUNIOC_PING;
+      cmd = RPMSGIOC_PING;
       val = (unsigned long)&ping;
     }
+#endif
+  else if (rpmsg_cb && rpmsg_cb(&cmd, &val, argv) == OK)
+    {
+      /* Nothing */
+    }
   else
     {
       nsh_output(vtbl, g_fmtarginvalid, argv[1]);
@@ -580,6 +585,102 @@ static int cmd_rptun_once(FAR struct nsh_vtbl_s *vtbl,
   return cmd;
 }
 
+static int cmd_rpmsg_recursive(FAR struct nsh_vtbl_s *vtbl,
+                               FAR const char *dirpath,
+                               FAR struct dirent *entryp,
+                               FAR void *pvarg)
+{
+  FAR char *path;
+  int ret = ERROR;
+
+  if (DIRENT_ISDIRECTORY(entryp->d_type))
+    {
+      return 0;
+    }
+
+  path = nsh_getdirpath(vtbl, dirpath, entryp->d_name);
+  if (path)
+    {
+      ret = cmd_rpmsg_once(vtbl, path, pvarg, NULL);
+      free(path);
+    }
+
+  return ret;
+}
+
+static int cmd_rpmsg_help(FAR struct nsh_vtbl_s *vtbl, int argc,
+                          FAR char **argv)
+{
+  nsh_output(vtbl, "%s <panic|dump> <path>\n", argv[0]);
+#ifdef CONFIG_RPMSG_PING
+  nsh_output(vtbl, "%s ping <path> <times> <length> <ack> "
+             "<period(ms)>\n\n", argv[0]);
+  nsh_output(vtbl, "<times>      Number of ping operations.\n");
+  nsh_output(vtbl, "<length>     The length of each ping packet.\n");
+  nsh_output(vtbl, "<ack>        Whether the peer acknowlege or "
+             "check data.\n");
+  nsh_output(vtbl, "             0 - No acknowledge and check.\n");
+  nsh_output(vtbl, "             1 - Acknowledge, no data check.\n");
+  nsh_output(vtbl, "             2 - Acknowledge and data check.\n");
+  nsh_output(vtbl, "<sleep(ms)>  Sleep interval between two operations.\n");
+#endif
+  nsh_output(vtbl, "<path>       Rpmsg device path.\n\n");
+  return OK;
+}
+
+int cmd_rpmsg(FAR struct nsh_vtbl_s *vtbl, int argc, FAR char **argv)
+{
+  if (argc >= 2 && strcmp(argv[1], "-h") == 0)
+    {
+      nsh_output(vtbl, "usage:\n\n");
+      return cmd_rpmsg_help(vtbl, argc, argv);
+    }
+
+  if (argc < 3)
+    {
+      nsh_output(vtbl, g_fmtargrequired, argv[0]);
+      return ERROR;
+    }
+
+  if (strcmp(argv[2], "all") == 0)
+    {
+      return nsh_foreach_direntry(vtbl, "rpmsg", "/dev/rpmsg",
+                                  cmd_rpmsg_recursive, argv);
+    }
+
+  return cmd_rpmsg_once(vtbl, argv[2], argv, NULL);
+}
+#endif
+
+/****************************************************************************
+ * Name: cmd_rptun
+ ****************************************************************************/
+
+#if defined(CONFIG_RPTUN) && !defined(CONFIG_NSH_DISABLE_RPTUN)
+static int cmd_rptun_cb(FAR int *cmd, FAR unsigned long *val,
+                        FAR char **argv)
+{
+  if (strcmp(argv[1], "start") == 0)
+    {
+      *cmd = RPTUNIOC_START;
+    }
+  else if (strcmp(argv[1], "stop") == 0)
+    {
+      *cmd = RPTUNIOC_STOP;
+    }
+  else if (strcmp(argv[1], "reset") == 0)
+    {
+      *val = atoi(argv[3]);
+      *cmd = RPTUNIOC_RESET;
+    }
+  else
+    {
+      return ERROR;
+    }
+
+  return OK;
+}
+
 static int cmd_rptun_recursive(FAR struct nsh_vtbl_s *vtbl,
                                FAR const char *dirpath,
                                FAR struct dirent *entryp,
@@ -596,7 +697,7 @@ static int cmd_rptun_recursive(FAR struct nsh_vtbl_s *vtbl,
   path = nsh_getdirpath(vtbl, dirpath, entryp->d_name);
   if (path)
     {
-      ret = cmd_rptun_once(vtbl, path, pvarg);
+      ret = cmd_rpmsg_once(vtbl, path, pvarg, cmd_rptun_cb);
       free(path);
     }
 
@@ -605,6 +706,14 @@ static int cmd_rptun_recursive(FAR struct nsh_vtbl_s *vtbl,
 
 int cmd_rptun(FAR struct nsh_vtbl_s *vtbl, int argc, FAR char **argv)
 {
+  if (argc >= 2 && strcmp(argv[1], "-h") == 0)
+    {
+      nsh_output(vtbl, "usage:\n\n");
+      nsh_output(vtbl, "rptun <start|stop> <path>\n");
+      nsh_output(vtbl, "rptun <reset> <path> <resetvalue>\n");
+      return cmd_rpmsg_help(vtbl, argc, argv);
+    }
+
   if (argc < 3)
     {
       nsh_output(vtbl, g_fmtargrequired, argv[0]);
@@ -617,7 +726,7 @@ int cmd_rptun(FAR struct nsh_vtbl_s *vtbl, int argc, FAR char **argv)
                                   cmd_rptun_recursive, argv);
     }
 
-  return cmd_rptun_once(vtbl, argv[2], argv);
+  return cmd_rpmsg_once(vtbl, argv[2], argv, cmd_rptun_cb);
 }
 #endif
 
@@ -631,6 +740,7 @@ int cmd_uname(FAR struct nsh_vtbl_s *vtbl, int argc, FAR char **argv)
   FAR const char *str;
   struct lib_memoutstream_s stream;
   struct utsname info;
+  struct utsname output;
   unsigned int set;
   int option;
   bool badarg;
@@ -721,8 +831,7 @@ int cmd_uname(FAR struct nsh_vtbl_s *vtbl, int argc, FAR char **argv)
   /* Process each option */
 
   first = true;
-  lib_memoutstream(&stream, alloca(sizeof(struct utsname)),
-                   sizeof(struct utsname));
+  lib_memoutstream(&stream, (FAR char *)&output, sizeof(output));
   for (i = 0; set != 0; i++)
     {
       unsigned int mask = (1 << i);
