@@ -33,7 +33,7 @@
 #endif
 
 #ifdef CONFIG_ADBD_NET_INIT
-#include "netutils/netinit.h"
+#  include "netutils/netinit.h"
 #endif
 
 /****************************************************************************
@@ -70,7 +70,7 @@ void adb_log_impl(int priority, FAR const char *func, int line,
   va_end(ap);
 }
 
-void adb_reboot_impl(const char *target)
+void adb_reboot_impl(FAR const char *target)
 {
 #ifdef CONFIG_BOARDCTL_RESET
   if (strcmp(target, "recovery") == 0)
@@ -94,6 +94,17 @@ int main(int argc, FAR char **argv)
 {
   adb_context_t *ctx;
 
+#ifdef CONFIG_ADBD_USB_BOARDCTL
+  struct boardioc_usbdev_ctrl_s ctrl;
+#  ifdef CONFIG_USBDEV_COMPOSITE
+  uint8_t usbdev = BOARDIOC_USBDEV_COMPOSITE;
+#  else
+  uint8_t usbdev = BOARDIOC_USBDEV_ADB;
+#  endif
+  FAR void *handle;
+  int ret;
+#endif
+
 #ifdef CONFIG_ADBD_BOARD_INIT
   boardctl(BOARDIOC_INIT, 0);
 #endif /* CONFIG_ADBD_BOARD_INIT */
@@ -102,15 +113,6 @@ int main(int argc, FAR char **argv)
 
   /* Setup USBADB device */
 
-  struct boardioc_usbdev_ctrl_s ctrl;
-#ifdef CONFIG_USBDEV_COMPOSITE
-  uint8_t usbdev = BOARDIOC_USBDEV_COMPOSITE;
-#else
-  uint8_t usbdev = BOARDIOC_USBDEV_ADB;
-#endif
-  FAR void *handle;
-  int ret;
-
   /* Perform architecture-specific initialization */
 
   ctrl.usbdev   = usbdev;
@@ -151,7 +153,7 @@ int main(int argc, FAR char **argv)
   ctx = adb_hal_create_context();
   if (!ctx)
     {
-      return -1;
+      return 1;
     }
 
   adb_hal_run(ctx);
